//Module Imports
const { resolve: resolvePath } = require("path");
const {
  existsSync: fexists,
  mkdirSync: mkdir,
  readdirSync: readdir,
  unlinkSync: fremove,
} = require("fs");
const config = require("../config.json");
const mongoose = require("mongoose");
mongoose.connect(
  `mongodb://${process.env.ALEXANDRIA_INTERNAL_URL}/nubian?authSource=admin`,
  {
    useUnifiedTopology: true,
    useNewUrlParser: true,
    user: "admin",
    pass: process.env.ALEXANDRIA_DATABASE_PASSWORD,
  }
);

const users = require("../schemas/user");
const files = require("../schemas/file");

function authorizedToView(userId, file) {
  if (`${file.owner}` == (userId = `${userId}`) || file.public) return true;
  if (file.view.includes(userId) || file.edit.includes(userId)) return true;
  return false;
}

function authorizedToEdit(userId, file) {
  return `${file.owner}` == `${userId}` || file.edit.includes(`${userId}`);
}

function getFile(userId, fileId) {
  return files.findOne({ _id: fileId }).then((file) => {
    if (authorizedToView(userId, file)) return file;
    return null;
  });
}

function deleteFiles(userId, fileIds) {
  return files.find({ _id: { $in: fileIds } }).then((databaseFiles) => {
    var failed = [];
    var toRemove = [];
    filesByOwner = {};
    databaseFiles.forEach((file) => {
      if (file.owner in filesByOwner) filesByOwner[file.owner].push(file);
      else filesByOwner[file.owner] = [file];
    });
    for (var owner in filesByOwner) {
      var deleteSize = 0;
      for (var i = filesByOwner[owner].length - 1; i >= 0; i--) {
        let file = filesByOwner[owner][i];
        if (!authorizedToEdit(userId, file)) {
          failed.push(`${file._id}`);
          filesByOwner[owner].splice(i, 1);
        } else deleteSize += file.size;
      }
      fileIds = fileIds.filter((fileId) => !failed.includes(fileId));
      toRemove.concat(
        databaseFiles.filter((file) => fileIds.includes(`${file._id}`))
      );
      // Update User's owned
      users
        .updateOne(
          { cairoId: owner, usedStorage: { $gte: deleteSize } },
          {
            $pull: { owned: { $in: fileIds } },
            $inc: {
              usedStorage: -deleteSize,
            },
          }
        )
        .exec();
      // Update files DB
      files.deleteMany({ _id: { $in: fileIds } }).exec();
      console.log(toRemove);
    }
    return { files: toRemove, failed };
  });
}

function publicfyFiles(userId, targetFiles) {
  return files.find({ _id: { $in: targetFiles } }).then((databaseFiles) => {
    var failed = [];
    databaseFiles.forEach((file) => {
      if (!authorizedToEdit(userId, file)) failed.push(`${file._id}`);
      else files.updateOne({ _id: file._id }, { public: !file.public }).exec();
    });
    return failed;
  });
}

function createUser(cairoId) {
  return users.create({
    cairoId,
    usedStorage: 0,
    storage: config.Storage.UserStorageSize * config.Storage.UserStorageUnit,
    owned: [],
    shared: [],
  });
}

function getUserByCairoId(cairoId) {
  return users.findOne({ cairoId }).then((user) => {
    if (!user) return createUser(cairoId);
    return user;
  });
}

function uploadFile(cairoId, fileData) {
  return getUserByCairoId(cairoId).then((user) => {
    if (user.usedStorage + fileData.size > user.storage) return null;
    return users
      .updateOne({ cairoId }, { $inc: { usedStorage: fileData.size } })
      .then(() => createFile(cairoId, fileData))
      .then((file) => {
        if (file == null) return null;
        users.updateOne({ cairoId }, { $push: { owned: file._id } }).then();
        return file;
      });
  });
}

function createFile(userId, fileData) {
  return files.create({
    path: fileData.path,
    owner: userId,
    name: fileData.originalname,
    date: fileData.filename.substring(0, fileData.filename.indexOf("-")),
    size: fileData.size,
    public: false,
    edit: [],
    view: [],
  });
}
module.exports = {
  deleteFiles,
  getFile,
  createUser,
  getUserByCairoId,
  publicfyFiles,
  uploadFile,
};
